#ifndef MC_PANEL_H
#define MC_PANEL_H

#include "OutputTreeNew.h"

#include <utility>
#include <string>

namespace jcs {
	class Converter;
	class DicomTreeCtrl;
	class OutputTreeCtrl;
}

class wxTreeEvent;
class wxSpinCtrl;

// Tooltip button
class ttButton: public wxButton
{
	public:
		ttButton(wxWindow* parent, wxWindowID window,
			const wxString& ButtonText, const wxString& HelpText)
			: wxButton(parent, window, ButtonText)
		{ SetToolTip(HelpText); }

};

// Define a new frame type
class McPanel : public wxPanel
{
public:
    // ctor(s)
    McPanel(wxWindow* parent);

	// dtor
	~McPanel();

    // event handlers
	void OnAddDir		(wxCommandEvent& event);
	void OnAddFiles		(wxCommandEvent& event);
	void OnRemove		(wxCommandEvent& event);
	void OnInfo			(wxCommandEvent& event);
	void OnDirectory	(wxCommandEvent& event);
	void OnNew			(wxCommandEvent& event);
	void OnConvert		(wxCommandEvent& event);
	void OnRename		(wxCommandEvent& event);
	void OnDicom		(wxCommandEvent& event);
	void OnImage		(wxCommandEvent& event);
	void OnOptions		(wxCommandEvent& event);

	void OnQuit			(wxCommandEvent& event);

	void OnContextMenu	(wxContextMenuEvent& event);

	void OnDropFiles	(wxDropFilesEvent& event);

	void OnChoiceOutput	(wxCommandEvent& event);
	void OnSourceSelect	(wxTreeEvent& event);
	void OnOutputSelect	(wxTreeEvent& event);
	void OnEditEnd		(wxTreeEvent& event);
	void OnEditBegin	(wxTreeEvent& event);

	void OnOverride		(const std::string& seriesUid);
	void OnMenuOverride	();

	bool GetSplitDirs() const { return m_split_dirs; }
	void SetSplitDirs(bool split);

	enum {
		COMMAND_REMOVE,
		COMMAND_DICOM,
		COMMAND_INFO,
		COMMAND_IMAGE,
		COMMAND_ADD_FILES,
		COMMAND_ADD_DIR,
		COMMAND_OPTIONS,
		COMMAND_DIRECTORY,
		COMMAND_NEW,
		COMMAND_CONVERT,
		COMMAND_RENAME,
		CHOICE_OUTPUT,
		SOURCE_CTRL,
		OUTPUT_CTRL
	};


protected:

    DECLARE_EVENT_TABLE()

private:

	void SetPathInfo(wxString path);
	jcs::DicomTreeCtrl* mpSourceCtrl;
	jcs::NewOutputTreeCtrl* mpOutputCtrl;

	ttButton* mpRemoveButton;
	ttButton* mpRenameButton;
	ttButton* mpOptionsButton;

	wxChoice* mpOutputChooser;
	int m_output_type;
	bool m_split_dirs;

	wxStaticText* mpPathInfo;
	
	jcs::Converter* mpConverter;

	void RefreshSourceCtrl();
	void ResetOutput();

	void UpdateOutputCtrl();
	void ShowMessageBox();

	friend class jcs::NewOutputTreeCtrl;

};

#endif

