#define _CRT_SECURE_NO_WARNINGS

#pragma warning(disable: 4996)

#include <string>
#include <vector>

#include "Basic3DOutputter.h"
#include "SeriesHandler.h"
#include "NiftiVolume.h"
#include "NiftiOutputter.h"

using namespace std;
using namespace jcs;

NiftiOutputterBase::NiftiOutputterBase(const Options& options) : 
Basic3DOutputter(options)
{
	if (rawExtension == _T("")) saveNii = true;
	else saveNii = false;
}

NiftiOutputter::NiftiOutputter() : 
NiftiOutputterBase(CreateOptions())
{
}

Options
NiftiOutputter::CreateOptions()
{
	Options options = Get3DOptions();
	options.pathname = "Nifti";
	return options;
}

BasicVolumeFormat* 
NiftiOutputterBase::GetOutputVolume(const char* file)
{
	return new NiftiVolume(file, headerExtension.mb_str(wxConvLocal), rawExtension.mb_str(wxConvLocal));
}

int
NiftiOutputter::ConvertSeries(SeriesHandler* handler)
{
	int bits_allocated, pixel_rep = 0;
	handler->Find("BitsAllocated", bits_allocated);
	handler->Find("PixelRepresentation", pixel_rep);

	switch (bits_allocated + pixel_rep) {

		case 9 : {
			NiftiConversion<wxInt8> conversion(this, handler);
			conversion.Convert();
				 }
			break;

		case 8 :{
			NiftiConversion<wxUint8> conversion(this, handler);
			conversion.Convert();
				}
			break;

		case 17 :{
			NiftiConversion<wxInt16> conversion(this, handler);
			conversion.Convert();
				 }
			break;

		case 16 :
		default :{
			NiftiConversion<wxUint16> conversion(this, handler);
			conversion.Convert();
				 }
	}

	return 1;
}

void
NiftiOutputterBase::SetSaveNii(bool value)
{
	saveNii = value;
	if (saveNii) {
		headerExtension = _T("nii");
		rawExtension = _T("");
	}
	else {
		headerExtension = _T("hdr");
		rawExtension = _T("img");
	}
}


void
NiftiOutputterBase::SetOption(const std::string& name, bool value)
{
	Basic3DOutputter::SetOption(name, value);
	if (name.find("nii") != string::npos) SetSaveNii(value);

}
